#include "ntddk.h"

#define NT_DEVICE_NAME L"\\Device\\NTName"
#define WIN32_DEVICE_NAME L"\\DosDevices\\MYDRIVER"
#define IOCTL_READ CTL_CODE  (FILE_DEVICE_UNKNOWN, 0x800, METHOD_BUFFERED, FILE_ANY_ACCESS) 
#define IOCTL_WRITE CTL_CODE (FILE_DEVICE_UNKNOWN, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS) 

NTSTATUS CtlCreate(IN PDEVICE_OBJECT, IN PIRP); 
NTSTATUS CtlClose(IN PDEVICE_OBJECT, IN PIRP); 
NTSTATUS CtlDispatch(IN PDEVICE_OBJECT,IN PIRP); 
VOID UnloadOperation(IN PDRIVER_OBJECT pDriverObject);

NTSTATUS DriverEntry(IN PDRIVER_OBJECT pDriverObject, IN PUNICODE_STRING RegistryPath)  
{
    PDEVICE_OBJECT pDeviceObject;
    UNICODE_STRING uniNtName;
    UNICODE_STRING uniWin32Name;
    RtlInitUnicodeString(&uniNtName, NT_DEVICE_NAME);
    RtlInitUnicodeString(&uniWin32Name, WIN32_DEVICE_NAME);  
    IoCreateSymbolicLink(&uniWin32Name, &uniNtName); 
    IoCreateDevice (pDriverObject, 0, &uniNtName, FILE_DEVICE_UNKNOWN, 0,FALSE,&pDeviceObject); 
    pDriverObject->MajorFunction[IRP_MJ_CREATE]=CtlCreate;
    pDriverObject->MajorFunction[IRP_MJ_CLOSE]=CtlClose;
    pDriverObject->MajorFunction[IRP_MJ_DEVICE_CONTROL]=CtlDispatch;
    pDriverObject->DriverUnload = UnloadOperation;

    return STATUS_SUCCESS;
}

NTSTATUS CtlCreate(IN PDEVICE_OBJECT pDeviceObject,IN PIRP Irp)
{
    Irp->IoStatus.Status=STATUS_SUCCESS;
    Irp->IoStatus.Information=0; 
    IoCompleteRequest(Irp,IO_NO_INCREMENT);
    return STATUS_SUCCESS;
}     

NTSTATUS CtlClose(IN PDEVICE_OBJECT pDeviceObject,IN PIRP Irp)
{
    Irp->IoStatus.Status=STATUS_SUCCESS;
    Irp->IoStatus.Information=0;  
    IoCompleteRequest(Irp,IO_NO_INCREMENT);
    return STATUS_SUCCESS;   
}

VOID UnloadOperation(IN PDRIVER_OBJECT pDriverObject)
{
    PDEVICE_OBJECT  deviceObject = pDriverObject->DeviceObject;
    UNICODE_STRING  uniWin32NameString;
    RtlInitUnicodeString( &uniWin32NameString, WIN32_DEVICE_NAME );
    IoDeleteSymbolicLink( &uniWin32NameString );    
    IoDeleteDevice( deviceObject ); 
    return;
}

NTSTATUS CtlDispatch(IN PDEVICE_OBJECT pDeviceObject,IN PIRP Irp)
{
    PIO_STACK_LOCATION pIrpStack;
    PUSHORT pIOBuffer; 

    USHORT Port;
    USHORT ValueToPort;

    pIrpStack=IoGetCurrentIrpStackLocation(Irp);
    pIOBuffer=Irp->AssociatedIrp.SystemBuffer;               

    switch (pIrpStack->Parameters.DeviceIoControl.IoControlCode)
    {

        case IOCTL_READ:
            Port=pIOBuffer[0];
            pIOBuffer[0]=READ_PORT_UCHAR((PUCHAR)Port);
            Irp->IoStatus.Information=2;
            break; 

        case IOCTL_WRITE:
            Port=pIOBuffer[0];
            ValueToPort=pIOBuffer[1];
            WRITE_PORT_USHORT((PUSHORT)Port,(USHORT)ValueToPort);
            Irp->IoStatus.Information=0;
            break;

    }

    Irp->IoStatus.Status=STATUS_SUCCESS;
    IoCompleteRequest (Irp,IO_NO_INCREMENT); 
    return STATUS_SUCCESS;


}